@@ -37,7 +37,8 @@ test_that("we can conatenate meta elements with empty ListOfmeta elements", {
            meta(content="Carl", property="dc:creator"))
 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
+  expect_length(out, 2)
+  expect_true(all(sapply(out, is, "meta")))
   
 ## in any order 
   out <-  c(meta(content="example", property="dc:title"),
@@ -45,8 +46,9 @@ test_that("we can conatenate meta elements with empty ListOfmeta elements", {
             new("ListOfmeta"))
 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
-  
+  expect_length(out, 2)
+  expect_true(all(sapply(out, is, "meta")))
+
 })
 
 
@@ -57,14 +59,16 @@ out <- c(meta(content="example", property="dc:title"),
   out <- c(out, meta("skos:note", "an editorial note"))
 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
-  
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+
   
   out <- c(meta("skos:note", "another editorial note"), out)
 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
-  
+  expect_length(out, 4)
+  expect_true(all(sapply(out, is, "meta")))
+
 })
 
 
@@ -74,8 +78,8 @@ test_that("we can concatenate two ListOfmeta elements", {
                 meta(content="Carl", property="dc:creator"))
   out <- c(metalist, metalist) 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
-  expect_equal(length(out), 4)
+  expect_length(out, 4)
+  expect_true(all(sapply(out, is, "meta")))
 })
 
 test_that("we can concatenate a ListOfmeta and a meta", {
@@ -85,15 +89,62 @@ test_that("we can concatenate a ListOfmeta and a meta", {
 
   out <- c(metalist, meta(content="a", property="b")) 
   expect_is(out, "ListOfmeta")
-  expect_is(out[[1]], "meta")
-  expect_equal(length(out), 3)
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
 
 })
 
+test_that("we can concatenate meta and ListOfmeta with ordinary lists", {
+
+  m1 <- meta(content="example", property="dc:title")
+  m2 <- meta(content="Carl", property="dc:creator")
+
+  out <- c(meta(content="a", property="b"), list(m1, m2)) 
+  expect_is(out, "ListOfmeta")
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+
+  out <- c(c(m1, m2), list(meta(content="a", property="b")))
+  expect_is(out, "ListOfmeta")
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+})
+
+test_that("NULLs are removed when concatenating meta", {
+
+  m1 <- meta(content="example", property="dc:title")
+  m2 <- meta(content="Carl", property="dc:creator")
+
+  out <- c(meta(content="a", property="b"), NULL, list(m1, m2)) 
+  expect_is(out, "ListOfmeta")
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+
+  out <- c(c(m1, m2), NULL, list(meta(content="a", property="b")))
+  expect_is(out, "ListOfmeta")
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+})
+
+test_that("must be list of lists, lists of meta, or meta when concatenating meta", {
+
+  m1 <- meta(content="example", property="dc:title")
+  m2 <- meta(content="Carl", property="dc:creator")
+
+  expect_error(c(m1, "foo"))
+  expect_error(c(c(m1, m2), "foo"))
+  expect_error(c(m1, list("foo")))
+  expect_error(c(c(m1, m2), list("foo")))
+  expect_silent(out <- c(m1, list(list(list(m1), m2))))
+  expect_is(out, "ListOfmeta")
+  expect_length(out, 3)
+  expect_true(all(sapply(out, is, "meta")))
+
+})
 
 test_that("we can read in a file with existing meta and append without overwriting", 
           {
-            f <- system.file("examples/biophylo.xml", package="RNeXML")
+            f <- system.file("examples", "biophylo.xml", package="RNeXML")
             nex <- nexml_read(f)
             g <- tempfile()
             nexml_write(nex, g)
